@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/client.c,v 11.3 2003/03/23 18:02:01 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/client.c,v 11.5 2006/01/18 22:51:25 jcb Exp $
  * client.c
  * Provides generic client support. It connects to the controller,
  * and maintains a game data structure in response to the Controller
@@ -17,25 +17,29 @@
  *                                                           *
  *************************************************************/
 
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/client.c,v 11.3 2003/03/23 18:02:01 jcb Rel $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/client.c,v 11.5 2006/01/18 22:51:25 jcb Exp $";
 
 #include "sysdep.h"
 #include <assert.h>
 #include "client.h"
 
-/* client_init: establish a connection to a controller */
-Game *client_init(char *address) {
+/* local for below two fns */
+static Game *_client_init(char *address, int reinit, int oldfd) {
   int fd;
   int i;
   Game *g;
 
-  if ( strcmp("-",address) == 0 ) {
-    fd = STDOUT_FILENO;
+  if ( reinit ) {
+    fd = oldfd;
   } else {
-    fd = connect_to_host(address);
-    if ( fd == INVALID_SOCKET ) {
-      perror("client_init: connect_to_host failed");
-      return 0;
+    if ( strcmp("-",address) == 0 ) {
+      fd = STDOUT_FILENO;
+    } else {
+      fd = connect_to_host(address);
+      if ( fd == (int)INVALID_SOCKET ) {
+	perror("client_init: connect_to_host failed");
+	return 0;
+      }
     }
   }
 
@@ -59,10 +63,21 @@ Game *client_init(char *address) {
   }
   
   g->fd = fd;
-  g->cseqno = 0;
+  if ( ! reinit ) g->cseqno = 0;
   return g;
 }
 
+/* client_init: establish a connection to a controller */
+Game *client_init(char *address) {
+  return _client_init(address,0,0);
+}
+
+/* client_reinit: as above, but use the existing fd or handle passed
+   as an argument */
+Game *client_reinit(int fd) {
+  return _client_init(NULL,1,fd);
+}
+
 /* client_connect: take an id and a name, and send a connect message.
    Return 1 on success, or 0 on failure. */
 int client_connect(Game *g, int id, char *name) {
@@ -82,20 +97,30 @@ int client_connect(Game *g, int id, char *name) {
   return 1;
 }
 
-/* close connection and free storage */
-Game *client_close(Game *g) {
+/* internal for below */
+static Game *_client_close(Game *g,int close) {
   int i;
 
-  close_socket(g->fd);
+  if ( close ) close_socket(g->fd);
   for ( i = 0; i < NUM_SEATS; i++ ) {
     set_player_name(g->players[i],NULL);
     free((void *)(g->players[i]));
   }
   free(g);
-  g->fd = INVALID_SOCKET;
   return NULL;
 }
 
+/* close connection and free storage */
+Game *client_close(Game *g) {
+  return _client_close(g,1);
+}
+
+/* free storage, but don't actually close connection */
+Game *client_close_keepconnection(Game *g) {
+  return _client_close(g,0);
+}
+
+
 /* client_send_packet: send the given packet out on the game fd.
    Return sequence number of packet. */
 int client_send_packet(Game *g, PMsgMsg *m) {
