@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/client.h,v 11.2 2003/03/23 18:02:21 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/client.h,v 11.3 2006/01/14 14:53:18 jcb Exp $
  * client.h
  * header file for client support routines.
  */
@@ -28,6 +28,10 @@
 */
 Game  *client_init(char *address);
 
+/* client_reinit: the same, but uses already established network
+   connection passed as fd/handle */
+Game *client_reinit(int fd);
+
 /* client_connect: take an id and a name, and send a connect message.
    Return 1 on success, or 0 on failure. */
 int client_connect(Game *g, int id, char *name);
@@ -37,6 +41,10 @@ int client_connect(Game *g, int id, char *name);
 */
 Game *client_close(Game *g);
 
+/* client_close_keepconnection: as above, but doesn't actually
+   close the connection */
+Game *client_close_keepconnection(Game *g);
+
 /* client_send_packet sends a packet to the controller.
    The return value is 0 on failure, or the sequence number of the packet 
    on success. N.B. If packets are sent bypassing this routine, the
