@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/game.c,v 11.25 2003/06/02 16:56:53 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/game.c,v 11.34 2006/01/13 17:57:00 jcb Exp $
  * game.c
  * Routines for manipulating game data structures.
  */
@@ -14,7 +14,7 @@
  *                                                           *
  *************************************************************/
  
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/game.c,v 11.25 2003/06/02 16:56:53 jcb Rel $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/game.c,v 11.34 2006/01/13 17:57:00 jcb Exp $";
 
 #include "game.h"
 #include "sysdep.h"
@@ -319,6 +319,9 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
     mjspecflags |= MJSevenPairs;
 
   switch ( cm->type ) {
+  case CMsgReconnect:
+    /* nothing to do - we'll be destroyed in a moment anyway */
+    return 0;
   case CMsgConnectReply:
     {
       CMsgConnectReplyMsg *m = (CMsgConnectReplyMsg *)cm;
@@ -1635,6 +1638,8 @@ int game_handle_cmsg(Game *g, CMsgMsg *cm)
       g->tile = m->tile;
       g->serial = m->discard;
       for (i=0;i<NUM_SEATS;i++) g->claims[i] = 0;
+      if ( game_flag(g,GFKong) ) { game_setflag(g,GFKongUponKong); }
+      game_setflag(g,GFKong);
       g->exposed_tile_count[m->tile] += 4;
       if ( chk ) set_danger_flags(g,p); /* the player may now be dangerous */
       return affected_id;
@@ -1987,7 +1992,6 @@ static void set_danger_flags(Game *g UNUSED, PlayerP p) {
 
 static void mark_dangerous_discards(Game *g) {
   PlayerP p;
-  unsigned int i;
   seats s,sup; /* seat of player, seat of supplier */
   Tile t; /* the discard claimed */
   unsigned int danger;
@@ -2068,6 +2072,7 @@ static void mark_dangerous_discards(Game *g) {
   }
 
   if ( danger && ! nochoice ) {
+    seats i;
     /* The supplier has supplied a dangerous discard without excuse */
     psetdflags(p,sup,danger);
     /* and this discharges the liability of any other player that
@@ -2082,4 +2087,121 @@ static void mark_dangerous_discards(Game *g) {
   if ( danger && nochoice ) game_setflag(g,GFNoChoice);
 }
 
+/* dump the game state in human-readable form into a large internal
+   buffer and return it.
+   This is intended for use in possibly dire situations, so it doesn't
+   do any memory management. As a service to the caller, it returns in the 
+   second argument (if non-null) the number of spare bytes in the returned 
+   buffer (counting from after the string terminator)
+*/
+char *game_print_state(Game *g, int *bytes_left) {
+  static char buf[4096];
+  static char claims[NUM_SEATS*15], ready[NUM_SEATS*3], flags[128];
+  static char walltiles[MAX_WALL_SIZE*4];
+  int i;
+  int n; char *b;
+  int tot;
+
+  if ( g == NULL ) { strcat(buf,"NO GAME\n"); return buf; }
+
+  claims[0] = 0; ready[0] = 0; flags[0] = 0;
+  for (i=0; i<NUM_SEATS; i++) {
+    char buf[15];
+    strcat(claims,nullprotect(game_print_Claims(g->claims[i])));
+    strcat(claims," ");
+    sprintf(buf,"%d ",g->ready[i]);
+    strcat(ready,buf);
+  }
+  for ( i = 0; i < 32; i++ ) {
+    if ( game_flag(g,i) ) {
+      strcat(flags,nullprotect(game_print_GameFlags(i)));
+      strcat(flags," ");
+    }
+  }
+  walltiles[0] = 0;
+  for ( i = 0; i < g->wall.size; i++ ) {
+    strcat(walltiles,tile_code(g->wall.tiles[i]));
+    strcat(walltiles," ");
+  }
+
+  buf[0] = 0;
+  n = sprintf(buf,""
+"Game State:\n"
+"players: (see later)\n"
+"round: %s\n"
+"hands_as_east: %d\n"
+"firsteast: %d\n"
+"state: %s\n"
+"active: %d\n"
+"paused: %s\n"
+"player: %s\n"
+"whence: %s\n"
+"supplier: %s\n"
+"tile: %s\n"
+"needs: %s\n"
+"serial: %d\n"
+"claims: %s\n"
+"cpos: %s\n"
+"chowpending: %d\n"
+"mjpending: %d\n"
+"konging: %s\n"
+"ready: %s\n"
+"flags: %s\n"
+"wall.tiles: %s\n"
+"wall.live_used: %d\n"
+"wall.live_end: %d\n"
+"wall.dead_end: %d\n"
+"wall.size: %d\n"
+"cmsg_check: %d\n"
+"cmsg_err: %s\n"
+"protversion: %d\n"
+"manager: %d\n"
+"option_table: (see later)\n"
+"fd: %d\n"
+"cseqno: %d\n"
+"userdata: %p\n",
+    nullprotect(tiles_print_TileWind(g->round)),
+    g->hands_as_east,
+    g->firsteast,
+    nullprotect(game_print_GameState(g->state)),
+    g->active,
+    g->paused ? g->paused : "-no-",
+    nullprotect(game_print_seats(g->player)),
+    nullprotect(game_print_Whence(g->whence)),
+    nullprotect(game_print_seats(g->supplier)),
+    tile_code(g->tile),
+    nullprotect(game_print_Whence(g->needs)),
+    g->serial,
+    claims,
+    nullprotect(player_print_ChowPosition(g->cpos)),
+    g->chowpending,
+    g->mjpending,
+    nullprotect(game_print_Konging(g->konging)),
+    ready,
+    flags,
+    walltiles,
+    g->wall.live_used,
+    g->wall.live_end,
+    g->wall.dead_end,
+    g->wall.size,
+    g->cmsg_check,
+    g->cmsg_err ? g->cmsg_err : "-none-",
+    g->protversion,
+    g->manager,
+    g->fd,
+    g->cseqno,
+    g->userdata
+    );
+  b = buf + n;
+  tot = n;
+  for ( i = 0; i < NUM_SEATS; i++ ) {
+    n = sprintf(b,"%s",g->players[i] ? player_print_state(g->players[i]) :
+      "NO PLAYER\n");
+    b += n;
+    tot += n;
+  }
+  if ( bytes_left ) *bytes_left = n-1;
+  return buf;
+}
+
 #include "game-enums.c"
