@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/greedy.c,v 11.6 2003/03/30 21:33:27 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/greedy.c,v 11.8 2006/01/14 14:58:14 jcb Exp $
  * greedy.c
  * This is a computer player. Currently offensive only.
  * Options not documented.
@@ -15,7 +15,7 @@
  *                                                           *
  *************************************************************/
  
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/greedy.c,v 11.6 2003/03/30 21:33:27 jcb Rel $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/greedy.c,v 11.8 2006/01/14 14:58:14 jcb Exp $";
 
 static int debugeval = 0;
 #include <stdlib.h>
@@ -100,7 +100,7 @@ static int rightdiscs[MaxTile]; /* disc_ser of last of each tile discarded */
 static int strategy_chosen;
 static int despatch_line(char *line);
 static void do_something(void);
-static void check_discard(PlayerP p,strategy *strat);
+static void check_discard(PlayerP p,strategy *strat,int closed_kong);
 static Tile decide_discard(PlayerP p, double *score, strategy *newstrat);
 static void update_tilesleft(CMsgUnion *m);
 static void maybe_switch_strategy(strategy *strat);
@@ -227,6 +227,10 @@ static int despatch_line(char *line) {
     break; /* damn all we can do */
   case CMsgGameOver:
     exit(0);
+  case CMsgReconnect:
+    /* we should never receive this */
+    warn("Received Reconnect command\n");
+    exit(1);
   case CMsgInfoTiles:
     /* We ignore these. */
     break;
@@ -333,16 +337,21 @@ static int despatch_line(char *line) {
     /* in this case, we need to do something else if it's not our turn! */
   case CMsgPlayerDiscards:
     if ( game_handle_cmsg(the_game,cm) != our_id && the_game->active)
-      check_discard(our_player,&curstrat);
+      check_discard(our_player,&curstrat,0);
     break;
     /* if this is us, we need to do something, and if it's 
        somebody else, we might be able to rob the kong */
   case CMsgPlayerDeclaresClosedKong:
+    if ( game_handle_cmsg(the_game,cm) == our_id && the_game->active)
+      do_something();
+    else if ( the_game->active ) 
+      check_discard(our_player,&curstrat,1); /* actually this checks the kong */
+    break;
   case CMsgPlayerAddsToPung:
     if ( game_handle_cmsg(the_game,cm) == our_id && the_game->active)
       do_something();
     else if ( the_game->active ) 
-      check_discard(our_player,&curstrat); /* actually this checks the kong */
+      check_discard(our_player,&curstrat,0); /* actually this checks the kong */
     break;
     /* In this case, it depends on the state of the game */
   case CMsgStartPlay:
@@ -399,7 +408,7 @@ static void do_something(void) {
   if ( the_game->state == Discarded ) {
     if ( the_game->player != our_seat
 	 && the_game->claims[our_seat] == UnknownClaim )
-      check_discard(our_player,&curstrat);
+      check_discard(our_player,&curstrat,0);
     return;
   }
 
@@ -566,8 +575,9 @@ static void do_something(void) {
 
 /* Check if we want the discard, and claim it.
    Arg is strategy.
-   Also called to check whether a kong can be robbed */
-static void check_discard(PlayerP p, strategy *strat) {
+   Also called to check whether a kong can be robbed -
+   the last arg says if being called on a closed kong */
+static void check_discard(PlayerP p, strategy *strat,int closedkong) {
   PMsgUnion m;
   double bestval,val;
   int canmj;
@@ -584,7 +594,8 @@ static void check_discard(PlayerP p, strategy *strat) {
        can be robbed. Since robbing a kong gets us an extra double,
        this is probably always worth doing, unless we're trying for
        some limit hand */
-    if ( player_can_mah_jong(p,the_game->tile,mjspecflags) ) {
+    if ( player_can_mah_jong(p,the_game->tile,mjspecflags)
+      && (!closedkong || player_can_thirteen_wonders(p,the_game->tile)) ) {
       m.type = PMsgMahJong;
       m.mahjong.discard = the_game->serial;
     } else {
