@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/gui.h,v 11.29 2003/04/16 22:28:58 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/gui.h,v 11.31 2006/01/14 14:53:31 jcb Exp $
  * gui.h
  * type defns and forward declarations for the gui module.
  */
@@ -170,6 +170,13 @@ typedef enum {
  DialogsBelow = 2,
  DialogsPopup = 3 } DialogPosition;
 
+/* Whether to file fault reports over the network */
+typedef enum {
+  DebugReportsUnspecified = 0,
+  DebugReportsNever = 1,
+  DebugReportsAsk = 2,
+  DebugReportsAlways = 3 } DebugReports;
+
 /* The rcfile is used to store assorted persistent data. We often
    need to read or update only selected parts of it. This set of
    flag bits identifies the groups that can be independently 
@@ -202,13 +209,15 @@ GtkWidget *build_or_refresh_option_panel(GameOptionTable *got,GtkWidget *panel);
 GtkWidget *build_or_refresh_prefs_panel(GameOptionTable *got,GtkWidget *panel);
 void button_set_tile(GtkWidget *b, Tile t, int ori);
 void chow_dialog_init(void);
-void close_connection(void);
+void close_connection(int keepconnection);
 void close_saving_posn(GtkWidget *w);
 void conc_callback(GtkWidget *w, gpointer data);
 void continue_dialog_init(void);
 void continue_dialog_popup(void);
+void control_server_processing(int on);
 void create_dialogs(void);
 void create_display(void);
+void debug_options_dialog_popup(void);
 void destroy_dialogs(void);
 void destroy_display(void);
 void dialog_popup(GtkWidget *dialog, DPPosn posn);
@@ -230,7 +239,7 @@ void messagewindow_init(void);
 void nag_popup(void);
 void option_reset_callback(GtkWidget *w, gpointer data);
 void option_updater_callback(GtkWidget *w, gpointer data);
-void open_connection(GtkWidget *w UNUSED, gpointer data);
+void open_connection(GtkWidget *w UNUSED, gpointer data, int fd);
 void open_dialog_init(char *idt, char *nt);
 void open_dialog_popup(GtkWidget *w UNUSED, gpointer data);
 void playing_prefs_init(void);
@@ -375,6 +384,9 @@ extern int playing_auto_declare_specials;
 extern int playing_auto_declare_losing;
 extern int playing_auto_declare_winning;
 
+/* debug options */
+extern DebugReports debug_reports; /* send debugging reports? */
+
 /* the player display areas */
 extern PlayerDisp pdisps[NUM_SEATS];
 
