@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/player.c,v 11.4.1.1 2004/05/09 21:49:56 jcb Exp $
+/* $Header: /home/jcb/newmj/RCS/player.c,v 11.8 2004/05/15 15:07:51 jcb Exp $
  * player.c
  * Implements functions on players.
  */
@@ -14,7 +14,7 @@
  *                                                           *
  *************************************************************/
 
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/player.c,v 11.4.1.1 2004/05/09 21:49:56 jcb Exp $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/player.c,v 11.8 2004/05/15 15:07:51 jcb Exp $";
 
 #define PLAYER_C /* so that PlayerP is not const */
 #include "player.h"
@@ -159,7 +159,11 @@ static int add_tile(PlayerP p, Tile t) {
 
   assert(!pflag(p,Hidden)); /* nonsense to call this on hidden player */
 
-  assert(p->num_concealed < MAX_CONCEALED);
+  //  assert(p->num_concealed < MAX_CONCEALED);
+  if ( ! (p->num_concealed < MAX_CONCEALED) ) {
+    error("player.c:add_tile(): player id %d has too many tiles: num_concealed = %d\n",p->id,p->num_concealed);
+    return 0;
+  }
   p->concealed[p->num_concealed++] = t;
   return 1;
 }
@@ -644,6 +648,7 @@ static int int_pd(PlayerP p, Tile t, int testonly) {
     }
     if ( testonly ) return 1;
     p->num_concealed -= 1;
+    pclearflag(p,NoDiscard);
     return 1;
   }
   if ( player_count_tile(p,t) < 1 ) {
@@ -754,15 +759,15 @@ int player_can_thirteen_wonders(PlayerP p, Tile d) {
   Player pcopy; /* we will manipulate this copy of the player */
   PlayerP pcp = &pcopy;
   int answer = 0;
-  int i, n;
+  int i, havetwo, n;
 
   if ( d != HiddenTile ) {
     copy_player(pcp,p);
     answer = (player_draws_tile(pcp,d)
 	      && player_can_thirteen_wonders(pcp,HiddenTile));
   } else {
-    int havetwo = 0;
     answer = 1;
+    havetwo = 0;
     for ( i=0; i < 13; i++ ) {
       n = player_count_tile(p,thirteen_wonders[i]);
       if ( n > 2 ) { answer = 0; break; }
@@ -1073,5 +1078,53 @@ char *tileset_string(TileSetP tp) {
   return buf;
 }
 
+/* print into an internal buffer a representation of the player */
+char *player_print_state(PlayerP p) {
+  static char buf[256],tiles[80],flags[80],dflags[80];
+  int i,j;
+
+  flags[0] = 0;
+  for ( i = 0; i < 32; i++ ) {
+    if ( pflag(p,i) ) {
+      strcat(flags,nullprotect(player_print_PlayerFlags(i)));
+      strcat(flags," ");
+    }
+  }
+  dflags[0] = 0;
+  for ( j = 0; j < NUM_SEATS; j++ ) {
+    for ( i = 0; i < 32; i++ ) {
+      if ( pdflag(p,j,1 << i) ) {
+	strcat(dflags,nullprotect(player_print_DangerSignals(1 << i)));
+	strcat(dflags," ");
+      }
+    }
+    strcat(dflags,"; ");
+  }
+  player_print_tiles(tiles,p,0);
+
+  sprintf(buf,"Player:\n"
+"id: %d\n"
+"name: %.100s\n"
+"wind: %s\n"
+"tiles: %s\n"
+"flags: %s\n"
+"dflags: %s\n"
+"cumulative_score: %d\n"
+"hand_score: %d\n"
+"err: %s\n"
+"userdata: %p\n",
+    p->id,
+    nullprotect(p->name),
+    nullprotect(tiles_print_TileWind(p->wind)),
+    tiles,
+    flags,
+    dflags,
+    p->cumulative_score,
+    p->hand_score,
+    nullprotect(p->err),
+    p->userdata);
+  return buf;
+}
+
 /* enum functions */
 #include "player-enums.c"
