@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/player.h,v 11.3 2003/02/19 20:59:30 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/player.h,v 11.4 2004/05/09 21:44:09 jcb Exp $
  * player.h
  * Contains datatypes representing the state of players.
  */
@@ -427,6 +427,10 @@ int player_swap_tile(PlayerP p, Tile oldtile, Tile newtile);
 */
 char *tileset_string(TileSetP tp);
 
+/* diagnostic function: print a text representation of the player's
+   state into a (possibly static) buffer and return it */
+char *player_print_state(PlayerP p);
+
 /* now import the functions for printing and scanning enums */
 #include "player-enums.h"
 
