@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/protocol.h,v 11.15 2003/10/06 19:58:19 jcb Rel $
+/* $Header: /home/jcb/newmj/RCS/protocol.h,v 11.17 2006/01/14 14:53:02 jcb Exp $
  * protocol.h
  * defines the messages passed between players and controller
  */
@@ -133,7 +133,7 @@ typedef struct _GameOptionEntry {
 */
 
 /* This define gives the protocol version implemented in this file */
-#define PROTOCOL_VERSION 1060
+#define PROTOCOL_VERSION 1070
 
 /* This global variable, found in protocol.c, is used by many functions
    to determine the protocol version currently being used.
@@ -161,6 +161,7 @@ typedef enum {
   CMsgInfoTiles = 1,
   CMsgStateSaved = 2,
   CMsgConnectReply = 10,
+  CMsgReconnect = 11,
   CMsgPlayer = 20,
   CMsgNewRound = 29,
   CMsgGame = 30,
@@ -215,6 +216,7 @@ typedef enum {
   PMsgSaveState = 1,
   PMsgLoadState = 2,
   PMsgConnect = 10,
+  PMsgRequestReconnect = 11,
   PMsgDeclareSpecial = 33,
   PMsgRequestPause = 36,
   PMsgReady = 37,
@@ -327,6 +329,24 @@ typedef struct _CMsgConnectReplyMsg {
   char *reason; /* human readable reason for refusal (if refused) */
 } CMsgConnectReplyMsg;
 
+/* This message requests a "reconnection": that is, the equivalent
+   of disconnecting and connection, without actually dropping the
+   network connection. If a positive reply is received, the client's
+   next act should be to resend its connect message.
+   This message was introduced at pversion 1070.
+*/
+typedef struct _PMsgRequestReconnectMsg {
+  PlayerMsgType type; /* PMsgRequestReconnect */
+} PMsgRequestReconnectMsg;
+
+/* This message grants a reconnection request.
+   This message was introduced at pversion 1070.
+*/
+typedef struct _CMsgReconnectMsg {
+  ControllerMsgType type; /* CMsgReconnect */
+} CMsgReconnectMsg;
+
+
 /* Message sent by controller to inform players of each other's existence.
    When a player connects, it receives one such message for every already
    existing player; and each existing player receives a message for the
