@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/sysdep.c,v 11.17 2004/04/16 06:33:06 jcb Exp $
+/* $Header: /home/jcb/newmj/RCS/sysdep.c,v 11.31 2006/01/22 21:02:37 jcb Exp $
  * sysdep.c
  * By intention, this file contains all functions that might need
  * fiddling with to cope with different variants of Unix.
@@ -21,15 +21,17 @@
  *                                                           *
  *************************************************************/
 
-static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/sysdep.c,v 11.17 2004/04/16 06:33:06 jcb Exp $";
+static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/sysdep.c,v 11.31 2006/01/22 21:02:37 jcb Exp $";
 
 #include "sysdep.h"
 #include <string.h>
 #include <fcntl.h>
+#include <errno.h>
 
 #ifndef WIN32
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <netinet/tcp.h>
 /* needed for HP-UX 11 with _XOPEN_SOURCE_EXTENDED */
 #include <arpa/inet.h>
 #include <sys/un.h>
@@ -50,20 +52,23 @@ static const char rcs_id[] = "$Header: /home/jcb/newmj/RCS/sysdep.c,v 11.17 2004
 
 /* warn: = fprintf(stderr,...), with automatic new line */
 /* log_msg is a generalized version with a syslog like warning
-   level. Currently only two levels are defined by the
-   LogLevel enum, LogWarning and LogInfo. */
+   level. */
 
 
 int (*log_msg_hook)(LogLevel l,char *);
 
-static char *log_prefixes[] = { "" , "-I- ", "+W+ " };
+char *(*log_msg_add_note_hook)(LogLevel l);
+
+static char *log_prefixes[] = { "" , "-I- ", "+W+ ", "*E* " };
 
 /* internal function */
 static int vlog_msg(LogLevel l, char *format,va_list args) {
   char buf[1024];
+  char *note = NULL;
   int ret;
   int n;
 
+  if ( l > LogError ) l = LogError;
   strcpy(buf,log_prefixes[l]);
   n = strlen(log_prefixes[l]);
   /* reserve 1 char for extra '\n' */
@@ -85,9 +90,20 @@ static int vlog_msg(LogLevel l, char *format,va_list args) {
 #else
   if ( buf[n-1] != '\n' ) strcat(buf,"\n");
 #endif
-  /* call the hook */
-  if ( log_msg_hook == NULL || log_msg_hook(l,buf) == 0 ) {
-    fprintf(stderr,"%s",buf);
+  /* add program supplied note? */
+  n = 0;
+  if ( log_msg_add_note_hook ) {
+    note = (*log_msg_add_note_hook)(l);
+    if ( note ) n = strlen(note);
+  }
+  {
+    char nbuf[1024+n+1];
+    strcpy(nbuf,buf);
+    if ( n > 0 ) { strcat(nbuf,note); }
+    /* call the hook */
+    if ( log_msg_hook == NULL || log_msg_hook(l,nbuf) == 0 ) {
+      fprintf(stderr,"%s",nbuf);
+    }
   }
   return ret;
 }
@@ -98,6 +114,15 @@ int log_msg(LogLevel l,char *format,...) {
   return vlog_msg(l,format,args);
 }
 
+int log_error(const char *function, const char *file, const int line, const char *format,...) {
+  char nformat[1024];
+  va_list args;
+  sprintf(nformat,"Error occurred in %s(), at %s:%d\n",function,file,line);
+  strmcat(nformat,format,1023-(strlen(nformat)+strlen(format)));
+  va_start(args,format);
+  return vlog_msg(LogError,nformat,args);
+}
+
 int warn(char *format,...) {
   va_list args;
   va_start(args,format);
@@ -184,7 +209,7 @@ int initialize_sockets(void *(*open_transform)(SOCKET),
 static int parse_address(const char *address,
 			 char *ret_host,
 			 unsigned short *ret_port,
-			 char *ret_file) {
+			 char *ret_file, size_t n) {
   if ( strchr(address,':') ) {
     /* grrr */
     if ( address[0] == ':' ) {
@@ -195,7 +220,7 @@ static int parse_address(const char *address,
     }
     return 0;
   } else {
-    if ( ret_file) strcpy(ret_file,address);
+    if ( ret_file) strmcpy(ret_file,address,n);
     return 1;
   }
 }
@@ -220,9 +245,9 @@ SOCKET set_up_listening_socket(const char *address) {
 
   unixsockets = 
 #ifdef WIN32
-    parse_address(address,name,&port,NULL);
+    parse_address(address,name,&port,NULL,0);
 #else
-    parse_address(address,name,&port,unaddr.sun_path);
+    parse_address(address,name,&port,unaddr.sun_path,sizeof(unaddr.sun_path));
 #endif /* WIN32 */
 
 #ifdef WIN32
@@ -319,7 +344,7 @@ SOCKET accept_new_connection(SOCKET fd) {
    The returned socket is marked close-on-exec .
 */
 
-SOCKET connect_to_host(const char *address) {
+static SOCKET _connect_to_host(int plain, const char *address) {
   SOCKET fd;
   struct sockaddr_in inaddr;
 #ifndef WIN32
@@ -334,13 +359,13 @@ SOCKET connect_to_host(const char *address) {
   if ( ! sockets_initialized ) initialize_sockets(NULL,NULL,NULL,NULL);
 
 #ifdef WIN32
-  unixsockets = parse_address(address,name,&port,NULL);
+  unixsockets = parse_address(address,name,&port,NULL,0);
   if ( unixsockets ) {
     warn("Unix sockets not supported on Windows");
     return INVALID_SOCKET;
   }
 #else
-  unixsockets = parse_address(address,name,&port,unaddr.sun_path);
+  unixsockets = parse_address(address,name,&port,unaddr.sun_path,sizeof(unaddr.sun_path));
 #endif /* WIN32 */
 
   if ( !unixsockets ) {
@@ -391,22 +416,57 @@ SOCKET connect_to_host(const char *address) {
   fcntl(fd,F_SETFD,1);
 #endif /* WIN32 */
 
-  if ( skt_open_transform ) {
+  /* Set a five-minute keepalive in order to dissuade natting routers
+     from dropping the connection */
+  { int data = 1;
+    int n;
+    n = setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (void *)&data, sizeof(int));
+    if ( n < 0 ) {
+      perror("connect_to_host: setsockopt keepalive failed");
+    } else {
+#ifdef TCP_KEEPIDLE
+      /* this call exists on modern Linuxes, but not everywhere */
+      data = 300;
+      n = setsockopt(fd, SOL_TCP, TCP_KEEPIDLE, &data, sizeof(int));
+      if ( n < 0 ) {
+	perror("connect_to_host: setsockopt keepidle failed");
+      }
+#endif
+    }
+  }
+
+  if ( !plain && skt_open_transform ) {
     return (SOCKET) skt_open_transform(fd);
   } else {
     return fd;
   }
 }
 
+SOCKET connect_to_host(const char *address) {
+  return _connect_to_host(0,address);
+}
+
+SOCKET plain_connect_to_host(const char *address) {
+  return _connect_to_host(1,address);
+}
+
+
 /* close a network connection */
-int close_socket(SOCKET s) {
-  if ( skt_closer ) {
+static int _close_socket(int plain, SOCKET s) {
+  if ( !plain && skt_closer ) {
     return skt_closer((void *)s);
   } else {
     return closesocket(s);
   }
 }
 
+int close_socket(SOCKET s) {
+  return _close_socket(0,s);
+}
+
+int plain_close_socket(SOCKET s) {
+  return _close_socket(1,s);
+}
 
 /* read a line, up to lf/crlf terminator, from the socket,
    and return it.
@@ -569,6 +629,45 @@ int fd_put_line(int fd,char *line) {
   return _put_line(fd,line,0);
 }
 
+int put_data(SOCKET fd, char *data, int len) {
+  int n;
+  
+  if ( len == 0 ) {
+#   ifdef WIN32
+    n = shutdown(fd, SD_SEND);
+#   else
+    n = shutdown(fd, SHUT_WR);
+#   endif
+    if ( n < 0 ) {
+      warn("put_data failed to shutdown: %s",strerror(errno));
+    }
+    return n;
+  }
+# ifdef WIN32
+  n = send(fd,data,len,0);
+# else
+  n = write(fd,data,len);
+# endif
+  if ( n < 0 ) {
+    warn("put_data failed: %s",strerror(errno));
+  }
+  return n;
+}
+
+int get_data(SOCKET fd, char *data, int len) {
+  int n;
+# ifdef WIN32
+  n = recv(fd,data,len,0);
+# else
+  n = read(fd,data,len);
+# endif
+  if ( n < 0 ) {
+    warn("put_data failed: %s",strerror(errno));
+  }
+  return n;
+}
+
+
 /* random integer from 0 to top inclusive */
 static int seed = 0;
 
@@ -588,6 +687,9 @@ void rand_seed(int s) {
   srand(seed = s);
 }
 
+/* utility function for following: return "NULL" for null string */
+char *nullprotect(char *s) { return s ? s : "NULL"; }
+
 /* feed a command to the system to be started in background.
    No fancy argument processing, just pass to shell or equiv.
    Return 1 on success, 0 on failure. */
@@ -598,7 +700,7 @@ int start_background_program(const char *cmd) {
   STARTUPINFO si;
   PROCESS_INFORMATION pi;
   
-  strncpy(buf,cmd,1023); buf[1023] = 0;
+  strmcpy(buf,cmd,1023); buf[1023] = 0;
   memset((void *)&si,0,sizeof(STARTUPINFO));
   si.cb = sizeof(STARTUPINFO);
   res = CreateProcess(NULL,buf,NULL,NULL,
@@ -611,7 +713,7 @@ int start_background_program(const char *cmd) {
   }
   return res;
 # else
-  strcpy(buf,cmd);
+  strmcpy(buf,cmd,sizeof(buf)-strlen(buf)-1);
   strcat(buf," &");
 # endif
   return ! system(buf);
@@ -703,7 +805,7 @@ int start_background_program_with_io(const char *cmd,int *childin,
     FALSE, DUPLICATE_SAME_ACCESS);
   CloseHandle(tochildw);
 
-  strncpy(buf,cmd,1023); buf[1023] = 0;
+  strmcpy(buf,cmd,1023); buf[1023] = 0;
   memset((void *)&si,0,sizeof(STARTUPINFO));
   si.cb = sizeof(STARTUPINFO);
   si.dwFlags = STARTF_USESTDHANDLES;
