@@ -1,4 +1,4 @@
-/* $Header: /home/jcb/newmj/RCS/sysdep.h,v 11.14 2004/04/16 06:32:59 jcb Exp $
+/* $Header: /home/jcb/newmj/RCS/sysdep.h,v 11.23 2006/01/22 20:42:04 jcb Exp $
  * sysdep.h
  * Miscellaneous system stuff that may require attention
  * by configuration procedures.
@@ -36,10 +36,6 @@ unsigned int sleep(unsigned int t);
 unsigned int usleep(unsigned int t);
 int getpid(void);
 int unlink(const char *f);
-/* snprintf is not in my Windows/Mingw. The "implementation" in
-   sysdep.c is NOT safe; it just ignores the length. */
-int vsnprintf(char *str, size_t size, const char *format, va_list ap);
-int snprintf(char *str, size_t size, const char *format, ...);
 #else
 #include <netinet/in.h>
 /* need this for gettimeofday */
@@ -83,7 +79,14 @@ typedef int SOCKET;
 /* warn is  fprintf(stderr,...) .
 */
 
-typedef enum { LogInfo = 1, LogWarning = 2 } LogLevel;
+typedef enum { LogInfo = 1, LogWarning = 2, LogError = 3 } LogLevel;
+
+/* If the global variable log_msg_add_note_hook is non-null,
+   it is called with the level, and returns a (char *) which
+   will be appended to the log message. The returned value should
+   be static, or at least long-lived enough.
+*/
+extern char *(*log_msg_add_note_hook)(LogLevel l);
 
 /* If the global variable log_msg_hook is non-null, it should
    be a pointer to a function which will be passed the formatted
@@ -92,17 +95,26 @@ typedef enum { LogInfo = 1, LogWarning = 2 } LogLevel;
 */
 extern int (*log_msg_hook)(LogLevel l,char *);
 
-int warn(char *format,...);
-
 /* log_msg is a generalized version with a syslog like warning
-   level. Currently only two levels are defined by the
-   LogLevel enum, LogWarning and LogInfo. */
+   level. */
 int log_msg(LogLevel l, char *format,...);
 
-/* and info is a log at info level */
+/* and here are abbreviations for the levels */
 int info(char *format,...);
 
-/* ignore the SIGPIPE signal. Return 0 on success, -1 on error */
+int warn(char *format,...);
+
+/* The error function is called log_error, because on error we
+   almost certainly want to add CPP information, which we pass in via
+   a macro */
+
+int log_error(const char *function, const char *file, const int line, const char *format,...);
+
+/* like this: */
+#define error(format, args...) log_error(__FUNCTION__,__FILE__,__LINE__,format,## args)
+
+/* ignore the SIGPIPE signal. Return 0 on success
+, -1 on error */
 int ignore_sigpipe(void);
 
 /* Initialize the socket library functions. This is only needed
@@ -153,8 +165,13 @@ SOCKET accept_new_connection(SOCKET fd);
 */
 SOCKET connect_to_host(const char *address);
 
-/* close a network connection */
+/* same as above, but ignores the socket transformers */
+SOCKET plain_connect_to_host(const char *address);
+
+
+/* close a network connection with or without transforms */
 int close_socket(SOCKET s);
+int plain_close_socket(SOCKET s);
 
 /* read a line, up to lf/crlf terminator, from the socket,
    and return it.
@@ -178,6 +195,11 @@ int put_line(SOCKET fd,char *line);
 char *fd_get_line(int fd);
 int fd_put_line(int fd, char *line);
 
+/* stuff arbitrary data down a socket. If len is zero, send EOF */
+int put_data(SOCKET fd, char *data, int len);
+/* and read it - data must be preallocated */
+int get_data(SOCKET fd, char *data, int len);
+
 /* rand_index: return a random integer from 0 to arg (inclusive).
  */
 unsigned int rand_index(int top);
@@ -196,6 +218,12 @@ void rand_seed(int seed);
    the terminating null.
 */
 #define strmcpy(dest,src,m) { strncpy(dest,src,m) ; dest[m-1] = '\000' ; }
+/* similarly */
+#define strmcat(dest,src,m) { strncat(dest,src,m) ; dest[m-1] = '\000' ; }
+
+/* another utility function: return arg, or "NULL" if it's null */
+char *nullprotect(char *s);
+
 
 /* feed a command to the system to be started in background.
    No fancy argument processing, just pass to shell or equiv
